#include "socket.h"
#include "type.h"
#include "netconf.h"    // Network Configuration
#include "sockutil.h"   // Useful function of W3100A
#include <reg51.h>
#include <irq52.h>
#include <stdio.h>
#include <kar.h>



//###############################################################################
//Define Part
//###############################################################################

IRQ_VECTOR(timer0int,TIMER0)
IRQ_VECTOR(NIC_IRQ,IRQ_INT0)
void Delay(u_int delay);
void InitNetConfig(void);

unsigned char received=0;   // indicates a reception on the channels
near  u_char addr[4];       // Variable to be used in network configuration and recvfrom()
near  u_long SpecifiedAddr;
near  u_char addrstr[16];
near  u_int port =5000;         // Variable to be used to store Destination Port in recvfrom()
near  int RecvSize;         // Variable to be used to store received data size
near  SOCKET s;               // Variable to be used to designate channel of W3100A
near  char IsFirstRecv = 0;       // Variable to be used to have been received data alreay. when it is set, Data is alreay received.
near  u_char * pRecvBuf[60];
near unsigned char tick;
sfr  CKCON      =   0x8F;
char IsISROK    =   0;          // if interrupt occurred then 1
near u_char ip[6];              // Variable to setup network information
extern signed char socket(SOCKET, u_char, u_int, u_char);
extern u_int recvfrom(SOCKET , const u_char * , u_int ,u_char * , u_int *);
extern u_int sendto(SOCKET , const u_char * , u_int , u_char * , u_int);
extern u_char  I_STATUS[4];              // Store Interrupt Status according to channels
extern u_int Local_Port;                // Designate Local Port
extern un_l2cval xdata SEQ_NUM;               // Set initial sequence number
extern void sysinit(void);
extern void initseqnum(SOCKET);
extern void wait_1us(int);
extern void wait_1ms(int);
extern void initW3100A(void);

/*********************************************************************************
*
*
* void WriteWr3100(unsigned int adr,unsigned char Byte)
**********************************************************************************/
#pragma option -w
void WriteWr3100(unsigned int adr,unsigned char Byte)
{
#asm
        push IE         ; save old IRQ mask
        clr EA          ; disable IRQ
        mov DPL,R7
        mov DPH,R6
        mov A,R5
        movx @DPTR,A
        pop IE
#endasm
}

/*********************************************************************************
* 
*
* ReadWr3100(unsigned int adr)
**********************************************************************************/
#pragma option -w
unsigned char ReadWr3100(unsigned int adr)
{
#asm
        push IE         ; save old IRQ mask
        clr EA          ; disable IRQ
        mov DPL,R7
        mov DPH,R6
        movx A,@DPTR
        mov R7,A
        pop IE
#endasm
}

//*****************************************************
// ####Install a vector to the function for Timer 0
// feuert alle 25ms
void timer0int (void) interrupt  
{
    TH0 = 106;   // teilt auf 40Hz
    tick--;
    if (tick ==0)
    {
      tick=40;
   //   P1_B0^=1;     //toggle LED
    } 
}

//********************************************************************************
//Description  : Interrupt service routine
//Argument     : NOne.
//Return Value : None.
//Note         :
//********************************************************************************
void NIC_IRQ (void) interrupt
{
    u_char status;
    EX0 = 0;                          // INT0 DISABLE
    status = INT_REG;
    while (status) 
    {
         if (status & 0x01) 
        {   // channel 0 interrupt(sysinit, sockinit, established, closed, timeout, send_ok, recv_ok)
            I_STATUS[0] = INT_STATUS(0);
//          if (I_STATUS[0] & SESTABLISHED) ISR_ESTABLISHED(0);
//          if (I_STATUS[0] & SCLOSED) ISR_CLOSED(0);
         INT_REG = 0x01;
        }
        if (status & 0x02) 
        {   // channel 1 interrupt(sysinit, sockinit, established, closed, timeout, send_ok, recv_ok)
            I_STATUS[1] = INT_STATUS(1);
//          if (I_STATUS[1] & SESTABLISHED) ISR_ESTABLISHED(1);
//          if (I_STATUS[1] & SCLOSED) ISR_CLOSED(1);
            INT_REG = 0x02;
        }
        if (status & 0x04) 
        {   // channel 2 interrupt(sysinit, sockinit, established, closed, timeout, send_ok, recv_ok)
            I_STATUS[2] = INT_STATUS(2);
            //if (I_STATUS[2] & SESTABLISHED) ISR_ESTABLISHED(2);
            //if (I_STATUS[2] & SCLOSED) ISR_CLOSED(2);
            INT_REG = 0x04;
        }
        if (status & 0x08) 
        {   // channel 3 interrupt(sysinit, sockinit, established, closed, timeout, send_ok, recv_ok)
            I_STATUS[3] = INT_STATUS(3);
//          if (I_STATUS[3] & SESTABLISHED) ISR_ESTABLISHED(3);
//          if (I_STATUS[3] & SCLOSED) ISR_CLOSED(3);
            INT_REG = 0x08;
        }
        if (status & 0x10) 
        // channel 0 receive interrupt
           {
            P1_B0^=1;     //toggle LED
        //ISR_RX(0);
            INT_REG = 0x10;
           }
        if (status & 0x20) 
        // channel 1 receive interrupt
        //ISR_RX(1);
            INT_REG = 0x20;
        if (status & 0x40) 
         // channel 2 receive interrupt
         //ISR_RX(2);
            INT_REG = 0x40;
        if (status & 0x80) 
        // channel 3 receive interrupt
        //ISR_RX(3);
            INT_REG = 0x80;
        status = INT_REG;
    }
    INT_REG = 0xFF;
    printf ("\nInt! %x",INT_REG);
    printf ("\nStatus %x",I_STATUS[0]);
    EX0 = 1;
}

//********************************************************************************
//Description  : Main function of direct interface test 
//Argument     : NOne.
//Return Value : None.
//Note         :
//********************************************************************************
void main(void)
{
    char IsVerifyOK = 1;
// Initialize 8051
    PCON|=128;      // Baudrate double
    SCON = 0x40;    //124; // 8 Bit UART - PC-compatible
    TH1=251;        //19200,8,n,1 bei 18,432MHz
    TH0 = 106; 
    TMOD=33;        // use timer 1 as baudrate generator
    TCON=64; 
    ET0 = 1;
    TR0 = 1;
    EX0 = 1;        //externer IRQ ein fr NIC
    IT0 = 1;        //und flankengesteuert 
    EA = 1;         //Enable general IRQs 
    tick=1;
	printf  ("\n\nSystem Start"); 
    initW3100A();           // Initialization of W3100A
    InitNetConfig();
    printf("\nInNetOK");  //kommt nicht bis hier...
    s = 0;              // Use channel #0 of W3100A
    printf ("\nSocket: %d",socket(s,SOCK_DGRAM,5000,0));    // Open UDP channel
    wait_1ms(100);
    printf ("\nSocketStatus: %x", select(s,0));
    while(1)                             // Ping-Pong Service Routine
    {
        RecvSize = select(s,SEL_RECV);   // When received data from port #5000, read the size and send it to peer's port 5000
        if(RecvSize > 0)
        {
            printf ("\nReceived: %d",RecvSize );   
            RecvSize = recvfrom(s,pRecvBuf,RecvSize,addr,&port);
            if(RecvSize==-1)                      // Read data as size as buffer received
            {
                printf("\n<<<< Receive Fail >>>>");
                break;
            }
            if(!IsFirstRecv)
            {
                SpecifiedAddr = *((u_long*)addr);
                IsFirstRecv = 1;
                printf ("\nreceive!");
            }
            inet_ntoa(addr,addrstr);
            if(IsFirstRecv && SpecifiedAddr == *((u_long*)addr))        // it can receive data from another peer, but it can't send received data to the peer.
            {
               //PutString(addrstr);PutString(" : Received Size = ");//PutITOA(RecvSize);
                RecvSize = sendto(s,pRecvBuf,RecvSize,addr,port);
                if(RecvSize == -1)                  // Loopback received data to destination : Ping-Pong
                {
                    printf("\n<<<< Send Fail >>>>");
                    break;
                }
                printf("\n Sent Size=%d", RecvSize);//PutITOA(RecvSize);PutStringLn("");
            }
            else                                        // if the board received data from another peer
            {
                printf("\nUnknown peer send 'PING' to the board :%d",addrstr);//PutString(addrstr);PutStringLn("");
            }
        }   
    }

}
//*********************************************************
//Delay function
//*********************************************************
void Delay(u_int delay)
{
    for(; delay > 0; delay--);
}

//*********************************************************
//Description   :  Initialize Network Information to be used by W3100
//Argument      :  
//Return Value  :  
//Note          :  
//*********************************************************
void InitNetConfig(void)
{   
    char i;
    ip[0] = 0x11; ip[1] = 0x12; ip[2] = 0x13; ip[3] = 0x14; ip[4] = 0x15; ip[5] = 0x16;     // ETC.
//setMACAddr(ip);     // Setup MAC
    for (i=0; i<6; i++)
        WriteWr3100 ((SRC_HA_PTR + i), ip[i]);

    ip[0] = 192; ip[1] = 168; ip[2] = 1; ip[3] = 10;    // VPN Env.                     
//setIP(ip);      // Setup source IP
    for (i=0; i<4; i++)
        WriteWr3100 ((SRC_IP_PTR + i), ip[i]);
    ip[3] = 1;
//setgateway(ip);     // Setup gateway address
    for (i=0; i<4; i++)
        WriteWr3100 ((GATEWAY_PTR + i), ip[i]);

    ip[0] = 255; ip[1] = 255; ip[2] = 255; ip[3] = 0;
//setsubmask(ip);     // Setup subnet mask 
    for (i=0; i<4; i++)
        WriteWr3100 ((SUBNET_MASK_PTR + i), ip[i]);
//*********Important: Initializes the module
    sysinit();
//GetGWAddress(addr);
/*
    printf("\nG/W IP: ");
    for (i=0; i<4; i++)
    {
      printf("%d",ReadWr3100(GATEWAY_PTR+i));
      if (i<3)
        printf(".");
    }
//    GetIPAddress(addr);

    printf("\nIP : "); 
    for (i=0; i<4; i++)
    {
      printf("%d",ReadWr3100(SRC_IP_PTR+i));
      if (i<3)
        printf(".");
    }
    printf("\nSN: "); 
    for (i=0; i<4; i++)
    {
      printf("%d",ReadWr3100(SUBNET_MASK_PTR+i));
      if (i<3)
        printf(".");
    }
*/

}





